import os
from kafka import KafkaProducer
from kafka.admin import KafkaAdminClient, NewTopic
from pingpongx.utils import sanitize_topic_name

TOPIC_NAME = os.getenv('TOPIC_NAME')
BROKER = 'kafka:9092'
DEFAULT_PARTITIONS = 3
DEFAULT_REPLICATION_FACTOR = 1

producer = KafkaProducer(
    bootstrap_servers='kafka:9092',  # Use the advertised listener
    retries=5,                       # Retry connecting to the broker
    request_timeout_ms=20000,        # Extend timeout for slow startup
    api_version=(0, 10, 1),          # API version
    metadata_max_age_ms=300000       # Cache metadata for 5 minutes
)


async def send_event(user_id: str, message: str):
    """Send an event to a Kafka topic."""
    try:
        topic_name = await create_topic_if_not_exists(TOPIC_NAME)
        future = producer.send(topic_name, value=message.encode('utf-8'))
        record_metadata = future.get(timeout=10)
        print(f"Message sent to topic {record_metadata.topic}, partition {record_metadata.partition}, offset {record_metadata.offset}")
        return True
    except Exception as e:
        print(f"Failed to send message to Kafka: {e}")
        return False


async def topic_exists(topic_name: str):
    admin_client = KafkaAdminClient(bootstrap_servers='kafka:9092', api_version=(0, 10, 1))
    topics = admin_client.list_topics()
    return topic_name in topics


async def create_topic_if_not_exists(topic_name):
    """Check if the topic exists and create it if necessary."""
    try:
        admin_client = KafkaAdminClient(bootstrap_servers='kafka:9092', api_version=(0, 10, 1))
        existing_topics = admin_client.list_topics()
        topic_name = sanitize_topic_name(topic_name)

        if topic_name not in existing_topics:
            print(f"Topic '{topic_name}' does not exist. Creating it...")

            # Create the topic
            topic = NewTopic(
                name=topic_name,
                num_partitions=DEFAULT_PARTITIONS,
                replication_factor=DEFAULT_REPLICATION_FACTOR
            )
            admin_client.create_topics([topic])
            print(f"Topic '{topic_name}' successfully created.")
        else:
            print(f"Topic '{topic_name}' already exists.")
        return topic_name
    except Exception as e:
        print(f"Failed to check or create topic '{topic_name}': {e}")
        return None



from pingpongx.services.firestore_service import get_user_preferences, update_user_preferences


class UserPreferences:

    def __init__(self, user_id=None, email=True, sms=True):
        self.user_id = user_id
        self.preferences = {"sms": sms, "email": email}

    async def get_preferences(self):
        """Get user preferences for notifications."""
        if self.user_id.strip() == "":
            return {"success": False, "message": "Somthing wrong with User Id"}

        user_id = self.user_id.strip().lower()
        preferences = await get_user_preferences(user_id)
        if not preferences:
            return {"success": False, "message": "User preferences not found"}
        return preferences

    async def update_preferences(self):
        """Update user preferences for notifications."""

        if self.user_id.strip() == "":
            return {"success": False, "message": "Somthing wrong with User Id"}
        if self.preferences == {}:
            preferences = {"sms": True, "email": True}

        user_id = self.user_id.strip().lower()
        success = await update_user_preferences(user_id, self.preferences)
        if not success:
            return {"success": False, "message": "Failed to update preferences"}
        return {"success": True, "message": "Preferences updated successfully"}


async def get_preferences_service(user_id: str):
    """API to get user preferences for notifications."""
    service = UserPreferences(user_id)
    return await service.get_preferences()


async def update_preferences_service(user_id: str, preferences: dict):
    """API to update user preferences for notifications."""
    service = UserPreferences(user_id, email=preferences.get("email", True), sms=preferences.get("sms", True))
    return await service.update_preferences()



from fastapi import FastAPI
from fastapi import Request
from fastapi.responses import HTMLResponse
from starlette.templating import Jinja2Templates
from pingpongx.routes.notification import router as notification_router
from pingpongx.routes.user_preferences import router as preferences_router
from pingpongx.routes.auth_routes import router as auth_router

docs_config = {
    "openapi_url": None,
    "docs_url": None,
    "redoc_url": None
}

app = FastAPI(
    title="PingPong",
    version="v0.1",
    description="You can send mail & sms notifications across the world using PingPong.",
    contact={"name": "Karan Kapoor", "email": "pingpongreply02@gmail.com"},
    swagger_ui_parameters={"defaultModelsExpandDepth": -1},
    **docs_config
    )

templates = Jinja2Templates(directory="py_pingpong/templates")


@app.get("/", response_class=HTMLResponse)
async def home(request: Request):
    return templates.TemplateResponse("index.html", {"request": request})

app.include_router(notification_router, prefix="/notifications")
app.include_router(preferences_router, prefix="/preferences")
app.include_router(auth_router, prefix="/auth")



import requests


class MailgunEmailService:
    """MAILGUN email service"""

    def __init__(self, mailgun_api_key, mailgun_domain, mailgun_email):
        self.mailgun_api_key = mailgun_api_key
        self.mailgun_domain = mailgun_domain
        self.mailgun_email = mailgun_email

    async def send_email(self, to_emails, subject, message):
        """Sends an email using the Mailgun API."""

        to_emails = to_emails.split(",")
        url = f"https://api.mailgun.net/v3/{self.mailgun_domain}/messages"
        data = {
            "from": self.mailgun_email,  # Your verified email address on Mailgun
            "to": to_emails,
            "subject": subject,
            "text": message
        }
        try:
            response = requests.post(
                url,
                auth=("api", self.mailgun_api_key),
                data=data
            )
            print("RES: ", response.text)
            if response.status_code == 200:
                print(f"Email sent successfully to {to_emails} with status code: {response.status_code}\n-----------------------------------\n")
                return True, response.status_code
            return False, response.status_code
        except requests.exceptions.RequestException as e:
            print(f"Error sending email: {e}")
            return False, 400




from twilio.rest import Client


class SmsService:

    def __init__(self, account_sid, auth_token, twilio_phone_number):
        self.account_sid = account_sid
        self.auth_token = auth_token
        self.twilio_phone_number = twilio_phone_number

    def send_sms(self, to_phones, message):
        """Sends an SMS using Twilio."""
        try:

            to_phones = to_phones.split(",")
            client = Client(self.account_sid, self.auth_token)
            for phone in to_phones:
                print(f"Sending SMS to {phone}")
                message = client.messages.create(
                    body=message,
                    from_=self.twilio_phone_number,
                    to=phone
                )
                print(f"SMS sent successfully to {phone}. Message SID: {message.sid}\n-----------------------------------\n")
            return True

        except Exception as e:
            print(f"Failed to send SMS: {e}")
            return False



import requests
import os

ONESIGNAL_APP_ID = os.getenv("ONESIGNAL_APP_ID")
ONESIGNAL_API_KEY = os.getenv("ONESIGNAL_API_KEY")


def send_push_notification(user_ids, title, message, data=None):
    """Sends push notifications using OneSignal."""
    url = "https://api.onesignal.com/notifications?c=push"

    headers = {
        "Content-Type": "application/json; charset=utf-8",
        "Authorization": f"Key {ONESIGNAL_API_KEY}"
    }

    payload = {
        "app_id": ONESIGNAL_APP_ID,
        "include_player_ids": user_ids,
        "headings": {"en": title},
        "contents": {"en": message},
        "data": data or {}
    }

    try:
        response = requests.post(url, json=payload, headers=headers)
        response.raise_for_status()
        print(f"Push notification sent successfully: {response.json()}")
        return response.json()
    except requests.exceptions.RequestException as e:
        print(f"Error sending push notification: {e}")
        return None



from fastapi import APIRouter
from pingpongx.services.auth_service import signup, login

router = APIRouter()

router.add_api_route('/login', login, methods=["POST"])
router.add_api_route('/signup', signup, methods=["POST"])


from pingpongx.services.email_service import MailgunEmailService
from kafka import KafkaConsumer
from pingpongx.utils import validate_email, validate_phone_number, sanitize_topic_name
from pingpongx.services.push_service import send_push_notification
from pingpongx.services.redis_service import get_from_queue, read_from_queue, delete_data_by_msg
from pingpongx.services.sms_service import SmsService
import json
import time
import os

TOPIC_NAME = os.getenv('TOPIC_NAME')
REDIS_TIME_LIMIT = os.getenv("REDIS_TIME_LIMIT")


async def process_message(user_id: str, message_data, username: str, mailgun_api_key=None, mailgun_domain=None, mailgun_email=None, twilio_account_sid=None, twilio_auth_token=None, twilio_phone_number=None):
    try:
        channel = message_data.get('channel')
        notification_message = message_data.get('message')
        success = False

        if not all(key in message_data for key in ['user_id', 'channel', 'message']):
            print(f"Malformed message: {message_data}")
            return False

        if "email" in channel and user_id and mailgun_api_key and mailgun_domain and mailgun_email:
            if validate_email(user_id) is False:
                print(f"Invalid email address: {user_id} as user_id")
                return False

            email_subject = f"PingPong from {username}"
            email_body = f"Notification sent to {user_id} via {channel}: {notification_message}"
            mailgun_instance = MailgunEmailService(mailgun_api_key, mailgun_domain, mailgun_email)
            send_email_success, send_email_status = await mailgun_instance.send_email(user_id, email_subject, email_body)
            if send_email_status == 200 and send_email_success:
                success = True

        if "sms" in channel and user_id and twilio_account_sid and twilio_auth_token and twilio_phone_number:
            if validate_phone_number(user_id) is False:
                print(f"Invalid phone number: {user_id} as user_id")
                return False
            sms_instance = SmsService(twilio_account_sid, twilio_auth_token, twilio_phone_number)
            success = sms_instance.send_sms(user_id, notification_message)

        if channel == "push":
            title = f"Notification for User {user_id}"
            body = notification_message
            response = send_push_notification([""], title, body)
            if response:
                print(f"Push notification sent successfully to user {user_id}")
                success = True
            else:
                print(f"Failed to send push notification to user {user_id}")

        return success
    except Exception as e:
        print(f"Failed to process message: {e}")
        return False


async def consume_notifications(receiver: str, sender: str, mailgun_api_key=None, mailgun_domain=None, mailgun_email=None, twilio_account_sid=None, twilio_auth_token=None, twilio_phone_number=None):
    """Continuously fetch and process messages from the Redis queue."""
    try:
        print("\n-----------------------------------\nAbout to consume messages from Redis queue")

        topic_name = sanitize_topic_name(TOPIC_NAME)
        consumer = KafkaConsumer(
            topic_name,  # Your Kafka topic name
            bootstrap_servers='kafka:9092',  # Kafka broker address
            group_id='notification-group',
            auto_offset_reset='earliest',  # To consume from the beginning
            enable_auto_commit=False,
            api_version=(0, 10, 1)
        )
        messages = await read_from_queue(username=sender)

        for message in messages:
            if message:
                message = json.loads(message)
                notification_timestamp = message.get("timestamp", 0)
                current_time = time.time()

                if current_time - notification_timestamp <= int(REDIS_TIME_LIMIT):
                    process_status = await process_message(user_id=receiver, message_data=message, username=sender, mailgun_api_key=mailgun_api_key, mailgun_domain=mailgun_domain, mailgun_email=mailgun_email, twilio_account_sid=twilio_account_sid, twilio_auth_token=twilio_auth_token, twilio_phone_number=twilio_phone_number)
                    if process_status:
                        redis_data = await get_from_queue(username=sender)
                        consumer.commit()
                    else:
                        return {"success": False, "message": "unable to consume notifications."}
                else:
                    print(f"Deleting messages from Redis that are {REDIS_TIME_LIMIT} seconds old")
                    await delete_data_by_msg(user_id=receiver, msg=message)
            else:
                print("No messages to process, waiting...\n-----------------------------------\n")

        return {"success": True, "message": "notifications consumed"}
    except Exception as e:
        print(f"Failed to process message: {e}")
        return {"success": False, "message": f"Failed to consume notifications due to {e}"}




from fastapi import Request
from pingpongx.services.auth_middleware import require_auth
from pingpongx.services.kafka_consumer import consume_notifications
from pingpongx.services.redis_service import add_to_queue
from pingpongx.services.kafka_producer import send_event
from pingpongx.services.firestore_service import save_notification_log, get_user_preferences, update_user_preferences
from pingpongx.services.user_preferences import UserPreferences
from pingpongx.utils import generate_notification_id
from pingpongx.utils import validate_phone_number, validate_email
import time
import os

MAILGUN_EMAIL = os.getenv("MAILGUN_EMAIL", "pingpongreply02@gmail.com")
MAILGUN_API_KEY = os.getenv("MAILGUN_API_KEY", "")
MAILGUN_DOMAIN = os.getenv("MAILGUN_DOMAIN", "")
TWILIO_ACCOUNT_SID = os.getenv("TWILIO_ACCOUNT_SID", "")
TWILIO_AUTH_TOKEN = os.getenv("TWILIO_AUTH_TOKEN", "")
TWILIO_PHONE_NUMBER = os.getenv("TWILIO_PHONE_NUMBER", "")


class PingPong:
    """PingPong notification service."""

    def __init__(self, sender=None, receiver=None, message="", channels=None, mailgun_api_key=None, mailgun_domain=None, mailgun_email=None, twilio_account_sid=None, twilio_auth_token=None, twilio_phone_number=None):
        self.sender = sender
        self.receiver = receiver
        self.message = message
        self.channels = channels
        self.mailgun_api_key = mailgun_api_key
        self.mailgun_domain = mailgun_domain
        self.mailgun_email = mailgun_email
        self.twilio_account_sid = twilio_account_sid
        self.twilio_phone_number = twilio_phone_number
        self.twilio_auth_token = twilio_auth_token

    async def send_notification(self):
        try:

            user_id = self.receiver
            username = self.sender
            message = self.message
            channel_list = self.channels
            message_sent_for_channel = []

            if not username or username.strip() == "":
                return {"success": False, "message": f"Please login and try again."}

            if user_id == "" or message == "" or len(channel_list) < 1 or user_id.strip() == "" or message.strip() == "":
                return {"success": False, "message": f"Invalid user_id or message: {user_id} and {message}"}

            username = username.strip().lower()
            user_id = user_id.strip().lower()
            message = message.strip()

            if username == user_id:
                return {"success": False, "message": f"You can't send notification to yourself!"}

            for i in channel_list:
                if i not in ["email", "sms"]:
                    return {"success": False, "message": f"Invalid channel: {i}. Choose from ['email', 'sms']"}

                if i == "email" and validate_email(user_id) is False:
                    return {"success": False, "message": f"Invalid email address: {user_id} as user_id"}

                if i == "sms" and validate_phone_number(user_id) is False:
                    return {"success": False, "message": f"Invalid phone number: {user_id} as user_id"}

                if i == "email" and (self.mailgun_email is None or self.mailgun_domain is None or self.mailgun_email is None):
                    return {"success": False, "message": f"Invalid or missing Mailgun credentials."}

                if i == "sms" and (self.twilio_auth_token is None or self.twilio_phone_number is None or self.twilio_account_sid is None):
                    return {"success": False, "message": f"Invalid or missing Twilio credentials."}

            for channel in channel_list:
                publish = await publish_notification(user_id=user_id, message=message, channel=channel, username=username)
                if publish.get("success") is True:
                    consume = await consume_notifications(receiver=user_id, sender=username, mailgun_api_key=self.mailgun_api_key, mailgun_domain=self.mailgun_domain, mailgun_email=self.mailgun_email, twilio_account_sid=self.twilio_account_sid, twilio_auth_token=self.twilio_auth_token, twilio_phone_number=self.twilio_phone_number)
                    if consume.get("success") is True:
                        message_sent_for_channel.append(channel)

            if message_sent_for_channel and len(message_sent_for_channel) > 0:
                return {"success": True, "message": f"Notification sent to {user_id}."}
            return {"success": False, "message": f"Notification failed to {user_id}."}
        except Exception as e:
            return {"success": False, "message": f"Notification failed due to :{e}"}


@require_auth
async def notify(request: Request, data: dict = None, username: str = None):
    """api method to send notifications"""
    try:
        sender = username
        if data is None:
            return {"success": False, "message": f"Please login and try again with valid payload."}

        receiver = data.get("user_id", "")
        message = data.get("message", "")
        channel_list = data.get("channel", [])
        service = PingPong(sender=sender, receiver=receiver, message=message, channels=channel_list, mailgun_api_key=MAILGUN_API_KEY, mailgun_domain=MAILGUN_DOMAIN, mailgun_email=MAILGUN_EMAIL, twilio_account_sid=TWILIO_ACCOUNT_SID, twilio_auth_token=TWILIO_AUTH_TOKEN, twilio_phone_number=TWILIO_PHONE_NUMBER)
        response = await service.send_notification()
        return response
    except Exception as e:
        return {"success": False, "message": f"Notification failed due to :{e}"}


async def publish_notification(user_id: str, message: str, channel: str, username: str):
    """Send a notification to a user via the specified channel."""

    try:
        user_preferences = await get_user_preferences(user_id)
        if not user_preferences:
            await update_user_preferences(user_id, {"email": True, "sms": True})

        preferences = user_preferences.get("preferences", {})
        if not preferences.get(channel, False):
            return {"success": False, "message": f"User {user_id} hasn't opted for {channel} notifications."}

        notification_data = {"user_id": user_id, "message": message, "channel": channel, "sent_by": username, "timestamp": time.time()}
        redis_status = await add_to_queue(user_id, notification_data)
        if redis_status:
            kafka_status = await send_event(user_id, f"Notification sent to {user_id} via {channel}")
            if kafka_status:
                await save_notification_log(user_id, {"id": generate_notification_id(), "message": message,"sent_by": username, "channel": channel})
                return {"success": True, "message": f"Notification queued successfully for channel: {channel}"}

        return {"success": False, "message": "Notification queued failed"}
    except Exception as e:
        return {"success": False, "message": f"Notification queued failed due to :{e}"}




from fastapi import APIRouter
from pingpongx.services import user_preferences

router = APIRouter()

router.add_api_route('/get/{user_id}', user_preferences.get_preferences_service, methods=["GET"])
router.add_api_route('/update/{user_id}', user_preferences.update_preferences_service, methods=["PUT"])



import redis
import json
import os


REDIS_HOST = os.getenv("REDIS_HOST", "redis")
REDIS_PORT = int(os.getenv("REDIS_PORT", 6379))
QUEUE_NAME = os.getenv("QUEUE_NAME", "notifications")

try:
    redis_client = redis.StrictRedis(host=REDIS_HOST, port=REDIS_PORT, decode_responses=True)
    redis_client.ping()  # Test connection
    print("Connected to Redis successfully")
except redis.ConnectionError as e:
    print(f"Failed to connect to Redis: {e}")


async def add_to_queue(user_id: str, data: dict, ttl=1800):
    """Add data to a Redis queue."""
    try:
        redis_client.lpush(QUEUE_NAME, json.dumps(data))
        redis_client.expire(QUEUE_NAME, ttl)
        print(F"Added data to Redis queue: {QUEUE_NAME} successfully")
        return True
    except redis.exceptions.ConnectionError as e:
        print(f"Failed to connect to Redis: {e}")
    except redis.exceptions.TimeoutError as e:
        print(f"Failed to connect to Redis: {e}")
    except Exception as err:
        print(f"Failed to add to Redis Queue due to: {err}")
    return False


async def get_from_queue(username: str):
    """Retrieve data from a Redis queue."""
    try:
        length = redis_client.llen(QUEUE_NAME)

        for _ in range(length):
            message = redis_client.lpop(QUEUE_NAME)  # Pop the first message from the queue
            if not message:
                break

            try:
                message_data = json.loads(message)
                if message_data.get("sent_by") == username:
                    return message  # Return the matching message

                redis_client.rpush(QUEUE_NAME, message)
            except json.JSONDecodeError:
                print(f"Failed to parse message: {message}")

        return None  # No matching message found
    except Exception as err:
        print(f"Failed to get from Redis Queue due to: {err}")
        return None


async def read_from_queue(username: str):
    try:
        all_messages = redis_client.lrange(QUEUE_NAME, 0, -1)  # Get all messages from the queue
        matching_messages = []

        for message in all_messages:
            try:
                message_data = json.loads(message)  # Parse the JSON data
                if message_data.get("sent_by") == username:
                    matching_messages.append(message)
            except json.JSONDecodeError:
                print(f"Failed to parse message: {message}")

        return matching_messages
    except Exception as err:
        print(f"Failed to read from Redis Queue due to: {err}")
        return []


def delete_data_by_user_id(user_id: str):
    """Deletes data from Redis based on the specified user ID."""
    try:
        not_present = False
        while True:
            item = redis_client.lpop(QUEUE_NAME)
            if not item:
                not_present = True
                break

            item_data = json.loads(item)
            if item_data['user_id'] == user_id:
                continue
            redis_client.rpush(QUEUE_NAME, item)
        return {"success": True, "message": f"Data for user {user_id} deleted successfully" if not not_present else f"No data is present with this user_id: {user_id}"}

    except Exception as err:
        return {"success": False, "message": f"Data for user {user_id} deleted failed due to {err}"}


async def delete_data_by_msg(user_id: str, msg: dict):
    """Deletes data from Redis based on the specified user ID."""
    try:
        while True:
            item = redis_client.lpop(QUEUE_NAME)
            if not item:
                break

            item_data = json.loads(item)
            if item_data == msg:
                continue
            redis_client.rpush(QUEUE_NAME, item)
        return True

    except Exception as err:
        return False



import requests
import os
import uuid
import re

ONESIGNAL_APP_ID = os.getenv('ONESIGNAL_APP_ID', "")


def generate_device_token():
    try:
        url = "https://onesignal.com/api/v1/players"

        payload = {
            "device_type": 5,
            "language": "en",
            "timezone": "-28800",
            "game_version": "1.1.1",
            "device_model": "iPhone5,1",
            "device_os": "15.1.1",
            "session_count": 600,
            "tags": {
                "first_name": "Jon",
                "last_name": "Smith",
                "level": "99",
                "amount_spent": "6000",
                "account_type": "VIP"
            },
            "amount_spent": "100.99",
            "playtime": 600,
            "notification_types": 1,
            "lat": 37.563,
            "long": 122.3255,
            "country": "US",
            "timezone_id": "Asia/Kolkata",
            "app_id": ONESIGNAL_APP_ID
        }
        headers = {
            "accept": "application/json",
            "Content-Type": "application/json"
        }

        response = requests.post(url, json=payload, headers=headers)
        return response.json()
    except Exception as e:
        print(e)


def generate_notification_id():
    return str(uuid.uuid4())


def validate_email(email):
    """Validates an email address using a regular expression."""
    email_regex = r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$'
    return re.match(email_regex, email) is not None


def validate_phone_number(phone_number):
    """Validates a phone number."""
    phone_regex = r"^\+(?:[0-9] ?){6,14}[0-9]$"
    return re.match(phone_regex, phone_number) is not None


def sanitize_topic_name(topic_name):
    # Replace "@" with "-" to make the topic name valid
    sanitized_topic_name = topic_name.replace('@', '-')
    return sanitized_topic_name



import os
from passlib.context import CryptContext
from pingpongx.models import SignupRequest, LoginRequest
from pingpongx.services.firestore_service import db
import datetime
import jwt
from pingpongx.utils import validate_phone_number, validate_email

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")
JWT_SECRET = os.getenv("JWT_SECRET")
JWT_ALGORITHM = os.getenv("JWT_ALGORITHM")


def generate_jwt(username: str):
    """Generate a JWT token with the username."""
    try:
        payload = {
            "username": username,
            "exp": datetime.datetime.utcnow() + datetime.timedelta(hours=1),  # Token expires in 1 hour
            "iat": datetime.datetime.utcnow()
        }
        return jwt.encode(payload, JWT_SECRET, algorithm=JWT_ALGORITHM)
    except Exception as e:
        print(e)
        return None


def hash_password(password: str) -> str:
    """Hash the password using bcrypt."""
    return pwd_context.hash(password)


def verify_password(plain_password: str, hashed_password: str) -> bool:
    """Verify a plain password against the hashed one."""
    return pwd_context.verify(plain_password, hashed_password)


def register_user(username: str, password: str):
    """Register a new user."""
    try:
        username = username.strip().lower()
        if "@" in username:
            valid_username = validate_email(username)
        else:
            valid_username = validate_phone_number(username)

        if username == "" or valid_username is None:
            return {"message": "Invalid credentials", "success": False}
        user_ref = db.collection("users").document(username)
        if user_ref.get().exists:
            return {"message": "User already exists", "success": False}

        hashed_password = hash_password(password)
        user_ref.set({
            "username": username,
            "password": hashed_password,
            "created_at": datetime.datetime.utcnow().isoformat()
        })
        return {"message": "User registered successfully", "success": True}
    except Exception as e:
        return {"message": str(e), "success": False}


def login_user(username: str, password: str):
    """Login user by verifying credentials."""
    try:
        username = username.strip().lower()
        if "@" in username:
            valid_username = validate_email(username)
        else:
            valid_username = validate_phone_number(username)

        if username == "" or valid_username is None:
            return {"message": "Username can't be empty", "success": False}
        user_ref = db.collection("users").document(username)
        user_data = user_ref.get()
        if not user_data.exists:
            return {"message": "Invalid credentials", "success": False}

        user_data = user_data.to_dict()
        if not verify_password(password, user_data["password"]):
            return {"message": "Invalid credentials", "success": False}

        token = generate_jwt(username)
        if token:
            return {"message": "Login successful", "username": username, "token": token, "success": True}
        return {"message": "Login failed!", "success": False}
    except Exception as e:
        return {"message": str(e), "success": False}


def signup(request: SignupRequest):
    """Signup API."""
    return register_user(request.username, request.password)


def login(request: LoginRequest):
    """Login API."""
    return login_user(request.username, request.password)



from google.cloud import firestore
import datetime
import os

db = firestore.Client.from_service_account_json("firebase_credentials.json")
USER_PREFERENCE = os.environ.get("FIRESTORE_USER_PREFERENCE", "user_preferences")
NOTIFICATION = os.getenv("FIRESTORE_NOTIFICATION", "notifications")


async def get_user_preferences(user_id: str):
    """Retrieve user notification preferences from Firestore."""
    try:
        doc = db.collection(USER_PREFERENCE).document(user_id).get()
        if doc.exists:
            return doc.to_dict()
        else:
            return {
                "user_id": user_id,
                "preferences": {"email": True, "sms": True},
                "last_updated": None
            }

    except Exception as e:
        print(f"Error retrieving user preferences: {e}")
        return None


async def save_notification_log(user_id: str, notification: dict):
    """Save a notification log to Firestore."""
    try:
        user_doc = db.collection(NOTIFICATION).document(user_id)
        if user_doc.get().exists:
            user_doc.update({"notifications": firestore.ArrayUnion([notification])})
        else:
            user_doc.set({
                "user_id": user_id,
                "notifications": [notification],
                "created_at": datetime.datetime.utcnow().isoformat()})
    except Exception as e:
        print(f"Error saving notification log: {e}")


async def update_user_preferences(user_id, preferences):
    """Update the notification preferences for a user."""
    try:
        doc_ref = db.collection(USER_PREFERENCE).document(user_id)
        doc_ref.set({
            "user_id": user_id,
            "preferences": preferences,
            "last_updated": datetime.datetime.utcnow().isoformat()
        })
        print(f"Preferences updated for user {user_id}")
        return True
    except Exception as e:
        print(f"Error updating user preferences: {e}")
        return False


async def delete_user_preferences(user_id):
    """Delete the notification preferences for a user."""
    try:
        db.collection(USER_PREFERENCE).document(user_id).delete()
        print(f"Preferences deleted for user {user_id}")
        return True
    except Exception as e:
        print(f"Error deleting user preferences: {e}")
        return False



import jwt
from fastapi import HTTPException, Request
from functools import wraps
import os

JWT_SECRET = os.getenv("JWT_SECRET", "pingpong_secret_key")
JWT_ALGORITHM = os.getenv("JWT_ALGORITHM", "HS256")


def verify_jwt(token: str):
    """Decode the JWT token to extract username."""
    try:
        decoded = jwt.decode(token, JWT_SECRET, algorithms=[JWT_ALGORITHM])
        return decoded["username"]
    except jwt.ExpiredSignatureError:
        raise HTTPException(status_code=401, detail="Token has expired")
    except jwt.InvalidTokenError:
        raise HTTPException(status_code=401, detail="Invalid token")


def require_auth(func):
    """Decorator to enforce authentication on endpoints."""
    @wraps(func)
    async def wrapper(*args, **kwargs):
        request: Request = kwargs.get("request")
        token = request.headers.get("Authorization")
        if not token or not token.startswith("Bearer "):
            raise HTTPException(status_code=401, detail="Missing or invalid Authorization header")

        username = verify_jwt(token.split(" ")[1])
        kwargs["username"] = username
        return await func(*args, **kwargs)
    return wrapper



from setuptools import setup, find_packages

setup(
    name="pingpongx",
    version="0.1.1",
    author="Karan Kapoor",
    author_email="kaykay9464769@gmail.com",
    description="A dynamic notification delivery system with Redis and Kafka integration.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/karan-kap00r/PingPong",  # GitHub repository link
    packages=find_packages(),
    install_requires=[
        "fastapi",
        "uvicorn",
        "redis",
        "kafka-python",
        "google-cloud-firestore",
        "requests",
        "pydantic",
        "Jinja2",
        "PyJWT",
        "passlib",
        "bcrypt",
        "twilio"
    ],
    python_requires=">=3.8",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
